<?php

namespace YdBc\Plugin;

use YdBc\BigCommerce\Core as Api;
use YdBc\Plugin\Helper;

class Controller {

    const CATALOG_QUERY_VAR = 'lightrcommerce-catalog-path';
    const API_QUERY_VAR = 'lightrcommerce-api';

    static $currentCatalogItem;

    static function registerTemplate() {
        add_filter('template_include', LIGHTRCOMMERCE_PATH . 'src/Plugin/templates/Catalog.php');
    }

    /**
     * Register the routes for the objects of the controller.
     */
    static function addRewriteRule() {
        $pageId = get_option('lightrcommerce_bc_catalog_page');

        if (!$pageId) {
            // no page set to use as catalog entry point
            return null;
        }

        $page = get_post($pageId);

        // see list of possible status: https://wordpress.org/documentation/article/post-status/
        $isValidPageStatus = $page->post_status == 'publish' || $page->post_status == 'private';

        if (!$page || !$isValidPageStatus) {
            // page does not exist
            return null;
        }

        add_rewrite_rule(
                '^' . $page->post_name . '/(.*)/?$',
                'index.php?page_id=' . $pageId . '&' . self::CATALOG_QUERY_VAR . '=$matches[1]',
                'top'
        );
        flush_rewrite_rules();

        add_rewrite_rule(
                '^api/(.*)/?$',
                'index.php?' . self::API_QUERY_VAR . '=$matches[1]',
                'top'
        );
    }

    /**
     * Add the query var lightrcommerce-catalog-path.
     */
    static function addQueryVars($vars) {
        $vars[] = self::CATALOG_QUERY_VAR;
        $vars[] = self::API_QUERY_VAR;
        return $vars;
    }

    /**
     * Add the template redirect.
     */
    static function catalogRedirect($query) {
        //if the query var lightrcommerce-catalog-path is set load the template
        if (self::isCatalogPage($query)) {
            add_filter('the_content', function ($content) use ($query) {
                try {
                    if (@$_GET['forceerror'] == 1) {
                        throw new \Exception('Forced error');
                    }

                    $catalogItem = self::loadCatalogItem($query);
                } catch (\Exception $ex) {
                    $args = compact('ex');

                    if ($ex->getCode() == 404) {
                        $catalogItem['html'] = Helper::loadPartial('not_found', $args);
                    } else {
                        $catalogItem['html'] = Helper::loadPartial('middleware_error', $args);
                    }
                }
                
                $assetsUrl = LIGHTRCOMMERCE_URL;

                return <<<EOL
<link rel="stylesheet" href="$assetsUrl/assets/css/lightrcommerce.css">
<div class="lightrcommerce-wrapper">{$catalogItem['html']}</div>
EOL;
            });
        }

        return $query;
    }

    static function apiRedirect($query) {
        if ($query->get(self::API_QUERY_VAR) && $query->is_main_query()) {
            $api_path = $query->get(self::API_QUERY_VAR);

            $endpoint = str_replace('/api/', '', $_SERVER['REQUEST_URI']);

            switch ($endpoint) {
                case 'search':
                    $response = \YdBc\BigCommerce\Core::callApi($endpoint, [
                                'method' => 'POST',
                                'headers' => [
                                    'accept' => 'application/json',
                                    'content-type' => 'application/json',
                                ],
                                'body' => file_get_contents('php://input'),
                    ]);

                    header('Content-Type: application/json');

                    if (stripos($_SERVER['HTTP_ACCEPT_ENCODING'], 'gzip') !== false) {
                        ob_start('ob_gzhandler');
                        echo json_encode($response);
                        ob_end_flush();
                    } else {
                        echo json_encode($response);
                    }
                    die();

                default:
                    $response = \YdBc\BigCommerce\Core::callApi($endpoint, [], false);
                    echo $response;
                    die();
                //                    var_Dump($response, $endpoint);die;
                //var_dump($endpoint, file_get_contents('php://input'), $_REQUEST, $_SERVER);
            }
        }
    }

    static function parseResponse(string $response) {
        $html = $response;

        $meta = [];

        $html = preg_replace('#<meta charset=".*?" */>#iu', '', $html);

        $total = preg_match_all(
                '#<meta (property|name)="(.*?)" content="(.*?)" ?/?>#msiu',
                $html,
                $matches
        );

        for ($i = 0; $i < $total; ++$i) {
            $meta[] = [$matches[1][$i], $matches[2][$i], $matches[3][$i]];
            $html = str_replace($matches[0][$i], '', $html);
        }

        if (preg_match('#<title>(.*?)</title>#', $html, $match)) {
            $title = $match[1];
            $html = str_replace($match[0], '', $html);
        } else {
            $title = null;
        }

        /* @var $catalogPage WP_Post */
        $catalogPage = get_post(get_option('lightrcommerce_bc_catalog_page'));

        $html = str_replace('__LIGHTRCOMMERCE_CATALOG_PATH__', $catalogPage->post_name, $html);

        return [
            'title' => $title,
            'meta' => $meta,
            'html' => $html,
        ];
    }

    static function loadCatalogItem(\WP_Query $query) {
        if (!self::$currentCatalogItem) {
            $response = Api::getOutput('lookup?path=/' . $query->get(self::CATALOG_QUERY_VAR));
            $parsed = self::parseResponse($response);

            self::$currentCatalogItem = $parsed;
        }

        return self::$currentCatalogItem;
    }

    static function bufferStart($html) {
        ob_start();
    }

    static function addGeneratorTag() {
        printf('<meta name="generator" content="LightR Commerce headless integration for WordPress, by YoungDogs." />');
    }

    static function isCatalogPage(\WP_Query $query): bool {
        return $query->get(self::CATALOG_QUERY_VAR) && $query->get('page_id') && $query->is_main_query();
    }

    /**
     * @global \WP_Query $wp_query
     */
    static function headBufferEnd() {
        global $wp_query;

        if (self::isCatalogPage($wp_query)) {
            $head = ob_get_contents();
            ob_end_clean();

            try {
                self::loadCatalogItem($wp_query);

                $newTitle = sprintf('<title>%s</title>', self::$currentCatalogItem['title']);
                $head = preg_replace('#<title>.*?</title>#', $newTitle, $head);

                // replace existing meta tags with those from the middleware
                foreach (self::$currentCatalogItem['meta'] as $meta) {
                    $pattern = sprintf(
                            '#<meta %s="%s" content=".*?".*?/?>#msiu',
                            preg_quote($meta[0]),
                            preg_quote($meta[1])
                    );
                    $newMeta = sprintf('<meta %s="%s" content="%s">', $meta[0], $meta[1], $meta[2]);

                    if (preg_match($pattern, $head)) {
                        $head = preg_replace($pattern, $newMeta, $head);
                    } else {
                        $head .= $newMeta;
                    }
                }
            } catch (\Exception $ex) {
                
            } finally {
                echo $head;
            }
        } else {
            ob_end_flush();
        }
    }

}
